package com.xilinx.sdk.terminal;

import java.util.HashMap;
import java.util.Map;

import jssc.SerialPortException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

/**
 * @author ravimish
 * 
 */
public class SDKTerminalView extends ViewPart {

	private Action connectToSerialPortAction;

	private Action disconnectSerialPortAction;
	
	private Action clearConsoleAction;

	Text writeText;

	//StyledText readOutputText;
	
	StyledText readOutputText;

	private ComboViewer mySerialPorts;

	ComboViewer mySendPostFix;

	private Button writeButton;

	private Button resetButton;

	private Button clearButton;

	private Composite parent;

	protected Map<SerialPortWrapper, SerialPortListener> mySerialConnections = new HashMap<SerialPortWrapper, SerialPortListener>();

	protected int myLastUsedIndex;

	protected boolean myAutoScroll;

	private Label portConnDataLabel;

	private String portConnectedMsg = "Click on + button to add a port to the terminal.";

	private SerialPortWrapper conectedPort;

	


	public SDKTerminalView() {
	}

	@Override
	public void createPartControl(Composite parent) {
		this.parent = parent;
		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent,
                "com.xilinx.sdk.docs.user.sdk_terminal_view");
		parent.setLayout(new GridLayout());

		GridLayout gl = new GridLayout(3, false);
		gl.marginHeight = 0;
		gl.marginWidth = 0;
		
		Composite fTop = new Composite(parent, SWT.NONE);
		fTop.setLayout(gl);
		fTop.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		GridData theGriddata;

		// register the combo as a Selection Provider
		getSite().setSelectionProvider(mySerialPorts);
		
		
		theGriddata = new GridData(SWT.FILL, SWT.CENTER, true, false);
		theGriddata.horizontalSpan = 3;
		theGriddata.verticalSpan = 2;
		
		portConnDataLabel = new Label(fTop, SWT.SINGLE | SWT.NONE);	
		portConnDataLabel.setLayoutData(theGriddata);
		portConnDataLabel.setText(portConnectedMsg);
		portConnDataLabel.setForeground(Display.getDefault().getSystemColor(
				SWT.COLOR_DARK_GRAY));
		
		readOutputText = new StyledText(fTop, SWT.BORDER | SWT.H_SCROLL
				| SWT.V_SCROLL);
		theGriddata = new GridData(SWT.FILL, SWT.FILL, true, true);
		theGriddata.horizontalSpan = 3;
		theGriddata.verticalSpan = 3;
		readOutputText.setLayoutData(theGriddata);
		readOutputText.setEditable(false);
		readOutputText.setDoubleClickEnabled(true);
		readOutputText.setText("");

		readOutputText.addListener(SWT.Modify, new Listener() {
			public void handleEvent(Event e) {
				readOutputText.setTopIndex(readOutputText.getLineCount() - 1);
			}
		});

		

		theGriddata = new GridData(SWT.FILL, SWT.CENTER, true, false);
		theGriddata.horizontalSpan = 1;
		theGriddata.verticalSpan = 2;

		writeText = new Text(fTop, SWT.SINGLE | SWT.BORDER);
		writeText.setLayoutData(theGriddata);
		writeText.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				if(e.keyCode == SWT.CR){
					if( getConnectedSerialPort()!=null){
						getConnectedSerialPort().write(writeText.getText(), "\n\r"); 
						writeText.setText("");						
					}
				}

			}
		});


		writeButton = new Button(fTop, SWT.BUTTON1);
		writeButton.setText("Send");
		theGriddata = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		theGriddata.horizontalSpan = 1;
		theGriddata.verticalSpan = 2;
		writeButton.setLayoutData(theGriddata);
		writeButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				writeText.setFocus();
				if( getConnectedSerialPort()!=null){
					getConnectedSerialPort().write(writeText.getText(), "\n\r"); // System.getProperty("line.separator"));					
					writeText.setText("");
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		writeButton.setEnabled(true);

		clearButton = new Button(fTop, SWT.BUTTON1);
		clearButton.setText("Clear");
		theGriddata = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		theGriddata.horizontalSpan = 1;
		theGriddata.verticalSpan = 2;
		clearButton.setLayoutData(theGriddata);
		clearButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				readOutputText.setText("");
				clearButton.setEnabled(false);
				writeText.setFocus();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				// nothing needs to be done here
			}
		});
		clearButton.setEnabled(true);

		

		// make new log area

		// logConsole = SdkLogAppender.getLogConsole();
		// conoleViewer = new ScrollingTextConsoleViewer(parent, logConsole);
		// GridDataFactory.fillDefaults().grab(true,
		// true).applyTo(conoleViewer.getControl());

		//parent.getShell().setDefaultButton(writeButton);
		makeActions();
		contributeToActionBars();
	}

	@Override
	public void setFocus() {
		writeText.setFocus();
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		//fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(connectToSerialPortAction);
		manager.add(disconnectSerialPortAction);
		manager.add(clearConsoleAction);
		
	}


	/**
	 * listener calls this method to report activity back to view
	 * 
	 * @param stInfo
	 *            - info string
	 */
	public void reportActivity(String stInfo) {
		readOutputText.getText();
		readOutputText.append(stInfo);
		if (myAutoScroll) {
			readOutputText.setSelection(readOutputText.getCharCount());
		}

		clearButton.setEnabled(!readOutputText.getText().isEmpty());

		// update the connected data label
		// portConnDataLabel.setText(stInfo);
	}

	SerialPortWrapper getConnectedSerialPort() {
		return conectedPort;
	}

	private void makeActions() {
		connectToSerialPortAction = new Action() {
			@SuppressWarnings("synthetic-access")
			@Override
			public void run() {
				SerialConnectionDialog serDialog = new SerialConnectionDialog(
						parent.getShell());
				//serDialog.create();
				int open = serDialog.open();

				if (Window.OK == open && !serDialog.getSerialPort().isEmpty()) {
					connectSerial(serDialog.getSerialPort(),
							serDialog.getBaudRate(),serDialog.getDataBit(), serDialog.getStopBits(),serDialog.getParity());
				}
			}
		};
		connectToSerialPortAction.setText("Connect to serial port");
		connectToSerialPortAction.setToolTipText("Connect to serial port.");
		connectToSerialPortAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_OBJ_ADD));

		disconnectSerialPortAction = new Action() {
			@Override
			public void run() {
				SerialPortWrapper newSerial = getConnectedSerialPort();
				if (newSerial != null) {
					SerialPortListener theListener = mySerialConnections
							.get(newSerial);
					mySerialConnections.remove(newSerial);
					newSerial.removeListener(theListener);
					if (newSerial.isConnected())
						newSerial.dispose();
					// theListener.dispose();
					// SerialPortsUpdated();
					portConnDataLabel.setText(portConnectedMsg);
					// disconnectSerialPortAction.setEnabled(getConnectedSerialPort()!=null
					// && getConnectedSerialPort().IsConnected());
				}
			}
		};
		disconnectSerialPortAction.setText("Disconnect from serial port");
		disconnectSerialPortAction
				.setToolTipText("Remove a serial port from the monitor.");
		disconnectSerialPortAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_ELCL_REMOVE));
		
		
		clearConsoleAction = new Action() {
			@SuppressWarnings("synthetic-access")
			@Override
			public void run() {
				readOutputText.setText("");
			}
		};
		clearConsoleAction.setText("Clear Console");
		clearConsoleAction.setToolTipText("Clear Console.");
		clearConsoleAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_ETOOL_CLEAR));
	}

	/**
	 * @param comPort
	 *            - COM port to be connected
	 * @param baudRate
	 *            - Baud rate
	 * 
	 *            Connects the port with given baud rate
	 */
	public void connectSerial(String comPort, int baudRate, int dataBits, float stopBits, char parity) {

		SerialPortWrapper newSerialPort = null;
		try {
			newSerialPort = new SerialPortWrapper(comPort, baudRate, stopBits, dataBits, parity);
			if( !newSerialPort.connectionMsg.isEmpty()){
				reportActivity(newSerialPort.connectionMsg);
			}

			if (newSerialPort.isConnected()) {
				SerialPortListener portListener = new SerialPortListener(this,
						0);
				newSerialPort.addListener(portListener);
				portListener.event(System.getProperty("line.separator")
						+ "Connected to " + comPort + " at " + baudRate
						+ System.getProperty("line.separator"));
				mySerialConnections.put(newSerialPort, portListener);

				// update the port connected message
				portConnDataLabel.setText("Connected to: Serial (  "
						+ newSerialPort.getPortName() + ", "
						+ newSerialPort.getRate() + ", "
						+ newSerialPort.getParity() + ", "
						+ newSerialPort.getDatabits() + " )");
				setSerialPort(newSerialPort);

				return;
			}
		} catch (SerialPortException e) {
			e.printStackTrace();
			
			// MessageDialog.open(MessageDialog.ERROR, parent.getShell(),
			// "Connection Problem", msg, 0);
		}

	}

	private void setSerialPort(SerialPortWrapper serialPort) {
		this.conectedPort = serialPort;
	}

	/**
	     * 
	     */
	public void ComboSerialChanged() {
		writeButton.setEnabled(mySerialPorts.toString().length() > 0);
		resetButton.setEnabled(mySerialPorts.toString().length() > 0);
		parent.getShell().setDefaultButton(writeButton);
	}

	@Override
	public void dispose() {
		SerialPortWrapper serialPort = getConnectedSerialPort();
		if ( serialPort!=null && serialPort.isConnected()) {
			serialPort.dispose();
		}
		super.dispose();
	}

}

// XSIP watermark, do not delete 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
